#!/usr/bin/python
# -*- coding: utf-8 -*-

#  Copyright 2009 John Roesler
#
#  This file is part of dupRemove.
#
#  dupRemove is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#   dupRemove is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with dupRemove.  If not, see <http://www.gnu.org/licenses/>.


# Alternative lisences available upon request. POC: John Roesler, jroesler@gmail.com

# Maintained by John Roesler (jroesler@gmail.com)

#Versions:
#0.0    31 June 2009

import os
import sys
import time
import datetime
import pickle
from optparse import OptionParser, SUPPRESS_HELP

from dupremove import dupsearch, restore
import fs.file

# --- Set up optparse -h man page ---
parser = OptionParser()
parser.add_option("-d", "--dir" , "--directory", dest = "path", help = "The directory to search for duplicates.", metavar = "DIR", default=os.getcwd())
parser.add_option("-b", "--db", "--db-file", dest = "db", help = "Use a saved database as the starting point. Also, save the database to this file when finished.", metavar = "DBFILE", default=None)
parser.add_option("--mode", dest = "mode", 
                  default="SEMI",
                  help="automation mode: NONE, SEMI, or AUTO [default: %default]. NONE (no automation) mode with ask you to manually resolve all duplicates (NOT recommended). SEMI (semi-automatic) mode will ask you to resolve duplicates only when DupRemove cannot determine if they are in fact duplicates. AUTO (full automation) mode will automatically resolve duplicates to the best of DupRemove's ability (not recommended in this version, but improvements to this mode are planned).")
parser.add_option("-v","--verbose", dest="verb", help="Prints progress messages (give an int in [0,2] to specify the level of verbosity)", default='0')
#parser.add_option("-vv","--doubleverbose", action="store_true", dest="verb2", help="Prints even more messages", default=False)
parser.add_option("-c",action="store_true", dest="copyright", help="Prints copyright and licensing information", default=False)
parser.add_option("-l","--level",dest="level",help="Sets the duplicate removal level (0-9). 0 is no change, 1 removes only those files that are definately duplicates, etc.", default='1')
parser.add_option("--restore",action="store_true", dest="restore",help="Restore a file that has been previously deleted as a duplicate.",default=False)
parser.add_option("--update","--update-files", dest="update", help="Save track information retrieved from the Internet to the files. 'ALL' does this automatically, 'SOME' will do this for any files you have manually selected an MBID for, 'ALL_ASK' will ask for every file, 'SOME_ASK' is the same as SOME, but it will ask for confirmation every time, and 'NONE' (default) will not update anything.", default='NONE')
parser.add_option("--player","--media-player", dest="player", help="You can enter a terminal command to run the media player of your choice. Just insert '%s' where the file name should go. Example (default): 'xterm -e \"mplayer \\\"%s\\\"\"'",default='xterm -e "mplayer \\\"%s\\\""')
(options, args) = parser.parse_args()

if (options.copyright):
    print "  Copyright 2009 John Roesler\n\n   disAMBI is free software: you can redistribute it and/or modify\n   it under the terms of the GNU General Public License as published by\n   the Free Software Foundation, either version 3 of the License, or\n   (at your option) any later version.\n\n   disAMBI is distributed in the hope that it will be useful,\n   but WITHOUT ANY WARRANTY; without even the implied warranty of\n   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n   GNU General Public License for more details.\n\n   You should have received a copy of the GNU General Public License\n   along with dupRemove in a file called COPYING.\n   If not, see <http://www.gnu.org/licenses/>.\n\n\n Alternative lisences available upon request. POC: John Roesler, jroesler@gmail.com\n\n Maintained by John Roesler (jroesler@gmail.com)\n\nVersions:\n0.0    31 June 2009"
    sys.exit(0)
# Make some parameters conformant to their contracts.
# - the base path is stored without a seperator at the end
# - if the user changes the storage location for duplicates or good files, the option to move is automatically switched on. Enforces the description in the help page
if not (options.mode == "NONE" or options.mode == "SEMI" or options.mode == "AUTO"):
    print "Warning: Unrecognized mode: using SEMI"
    options.mode = "SEMI"
if options.db == None: options.db = "db"
try:
    options.level=int(options.level)
except ValueError:
    print "Warning: Unrecognized aggressiveness level: using 0"
    options.level=0
try:
    options.verb=int(options.verb)
except ValueError:
    print "Warning: Unrecognized verbosity level: using 0"
    options.verb=0
# Setting up db

def cleandb(db):
    for i in range(len(db)-1,-1,-1):
        if not db[i].verifyExists():
            del db[i]

errors=[]
if not os.path.exists(options.db):
    with open(options.db, 'w') as t:
        pickle.dump([], t)
try:   
    with open(options.db, 'r') as f:
        if (options.verb):
            print 'Loading database...'
        dups = pickle.load(f)
except Exception, e:
    errString = '\nError reading database file: '+os.path.abspath(options.db)+'. The file may be corrupted. Try deleteing or renaming the file to create a new database.'+'\n\t Error text: '+str(e)+'\nExiting...'
    print '\n'+errString
    errors.append(errString)
    quit()

cleandb(dups)
dups = sorted(dups,fs.file.sortOnPath)

logfile = 'log'+str(datetime.datetime.today().isoformat().replace(':','.'))+'.txt'
errorfile = 'errors'+str(datetime.datetime.today().isoformat().replace(':','.'))+'.txt'
gulag = os.path.realpath(options.path)+'.gulag'

if options.restore:
    restore.main(options.path,logfile,options.verb,dups)
else:
    dups = dupsearch.main(options.path,gulag,options.level,logfile,options.verb, options.mode, options.update, options.player, dups)

if options.verb: print 'Saving Database'
with open(options.db, 'w') as database:
    pickle.dump(dups,database)
if errors:
    with open(errorfile, 'w') as errorf:
        pickle.dump(errors,errorf)