'''
Created on Mar 21, 2010

@author: john
'''
# -*- coding: utf-8 -*-
#Main program file: dupremove.py 

#  Copyright 2009 John Roesler
#
#  This file is part of dupRemove.
#
#  dupRemove is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#   dupRemove is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with dupRemove.  If not, see <http://www.gnu.org/licenses/>.


# Alternative lisences available upon request. POC: John Roesler, jroesler@gmail.com

# Maintained by John Roesler (jroesler@gmail.com)

#Versions:
#0.0    31 June 2009

import os
import shutil
            

# clean() provides a somewhat standardized string for use in robust comparisons between files.
#In v0.0, this does the following:
# - replace all non-ascii chars with ascii chars
# - make the whole string upper-case
# - remove all instances of 'the' (to handle "Beatles"/"The Beatles" ambiguity
# - remove all non-alphanumeric characters
def clean(wordIn):
    word = wordIn.encode('ascii', 'replace')
    word = word.upper()
    word = word.replace("THE","")
    # TODO: fix this stupid implementation. What I want to do is iterate through word and append each alphanum to result. There must be some way to convert a char[] to a string.
    result = word[:]
    for ch in word:
        if not ch.isalnum():
            result = result.replace(ch,"")
    return str(result)

#___________________________
# The following section is for comparators. They allow sorting based on artist, track, etc.

def artistCmp(a, b):
    return cmp(a.artist, b.artist)

def albumCmp(a,b):
    return cmp(a.album, b.album)

def titleCmp(a, b):
    return cmp(a.title, b.title)

def sizeCmp(a, b):
    return cmp(a.size, b.size)

def rateCmp(a, b):
    return cmp(a.rate, b.rate)

def discCmp(a,b):
    return cmp(a.track[0], b.track[0])

def trackCmp(a,b):
    return cmp(a.track[1], b.track[1])
#__________________________

def sortOnPath(a,b):
    return cmp(a.getPath(), b.getPath())

# This poorly named function is used for sorting in order of file desirability. In v0.0, this means highest bitrate, then largest size.
def sort(a, b):
    tmp = rateCmp(a,b)*-1
    if (tmp!=0): return int(tmp) 
    tmp = sizeCmp(a,b)*-1
    if (tmp!=0): return int(tmp)
    return 0

# This function is for pretty-sorting LISTS of duplicates. It looks at the top record for each set and sorts by Artist, Album, Disc, and then Track.
def dupSortList(a,b):
    tmpA = a[0]
    tmpB = b[0]
    if artistCmp(tmpA,tmpB)!=0:
        comp = artistCmp(tmpA,tmpB)
    elif albumCmp(tmpA,tmpB)!=0:
        comp = albumCmp(tmpA,tmpB)
    elif discCmp(tmpA,tmpB)!=0:
        comp = discCmp(tmpA,tmpB)
    else:
        comp = trackCmp(tmpA,tmpB)
    return comp


# This function is for pretty-sorting sets of duplicates. It looks at the top record for each set and sorts by Artist, Album, Disc, and then Track.
def dupSort(a,b):
    # TODO: Wishing for a peek() method here. Chack again to see if one exists.
    tmpA = a.pop()
    tmpB = b.pop()
    if artistCmp(tmpA,tmpB)!=0:
        comp = artistCmp(tmpA,tmpB)
    elif albumCmp(tmpA,tmpB)!=0:
        comp = albumCmp(tmpA,tmpB)
    elif discCmp(tmpA,tmpB)!=0:
        comp = discCmp(tmpA,tmpB)
    else:
        comp = trackCmp(tmpA,tmpB)
    a.add(tmpA)
    b.add(tmpB)
    return comp

def pathIsFile(path,file):
    return path == file.getPath()
        
class File:

    def __init__(self, filepath, artFull=None, albFull=None, tr=None, titlFull=None, filesize=None, bitrate=None):
        self.path = [filepath]
        self.artist = clean(artFull)
        self.artistFull = artFull
        self.album = clean(albFull)
        self.albumFull = albFull
        self.track = tr
        self.title = clean(titlFull)
        self.titleFull = titlFull
        self.size = filesize
        self.rate = int(bitrate)
        self.links = []
        self.puid = None
        self.duration = None
        self.mbid = None
        self.toupdate=False
        
    def updatefile(self):
        import media.mp3
        media.mp3.updatemp3(self)
        
    def __str__(self):
        trackSt = str(self.track[0])
        discSt = str(self.track[1])
        if trackSt == 'None':
            trackSt = ''
        if discSt == 'None':
            discSt = ''
        return '\nPath:\t'+str(self.getPath())+'\nArtist:\t'+self.artistFull.encode('ascii','ignore')+'\nAlbum:\t'+self.albumFull.encode('ascii','ignore')+'\nTrack:\t'+trackSt+'\nDisc:\t'+discSt+'\nTitle:\t'+self.titleFull.encode('ascii','ignore')+'\nDuration:\t'+str(self.duration)+'\nSize:\t'+str(self.size)+'\nRate:\t'+str(self.rate)+'\nPUID:\t'+str(self.puid)+'\nMBID:\t'+str(self.mbid)+'\n'

    def setArtist(self,artFull):
        self.artist = clean(artFull)
        self.artistFull = artFull
        
    def setAlbum(self,albFull):
        self.album = clean(albFull)
        self.albumFull = albFull
        
    def setTitle(self,titlFull):
        self.title = clean(titlFull)
        self.titleFull = titlFull
        
    def getPath(self):
        return self.path[-1]

    def getPathList(self):
        return self.path

    def setPath(self, newPath):
        self.path.append(newPath)
        
    def popPath(self):
        oldpath = self.path.pop()
        if os.path.exists(os.path.dirname(self.getPath())):
            shutil.move(oldpath, self.getPath())
        else:
            self.setPath(oldpath)
            raise Exception('Destination path does not exist')
        
    def remove(self):
        self.path=None
    
    def verifyExists(self):
        ex = self.path and os.path.exists(self.getPath())
        if not ex:
            self.remove()
        return ex
    
    def getPuid(self):
        if self.puid == None:
            self.puid.getpuid(self.path)
        return self.puid